@@ -24,3 +24,12 @@
 
 /* for audio */
 #define AUDIO FALSE
+
+/* syntax flags to be used for REGEX matching */
+#define XLM_REGEX_SYNTAX RE_SYNTAX_EMACS
+
+
+
+/* to suppress autoconf warnings although not used: */
+/* #define HAVE_RX_H 1 */
+/* #define HAVE_GETOPT_H 1 */
