@@ -34,6 +34,7 @@
 
 #include "../config.h"
 #include "sysinc.H"
+#include "../import/import.h"
 #include "logclass.H"
 #include "menu.H"
 #include "extern.H"
@@ -41,7 +42,13 @@
 /*}}}*/
 
 /*{{{  Create Menu  */
-void fire_up_menu(GtkWidget* widget){
+/*
+  This function is called during initialisation to create
+  the popup menu 
+*/
+void 
+fire_up_menu(GtkWidget* widget)
+{
   // set up starting defaults for the menus:
   customizing = FALSE;
 
@@ -71,19 +78,19 @@ void fire_up_menu(GtkWidget* widget){
   // attach the callbacks:
   gtk_signal_connect_object( GTK_OBJECT(about), "activate",
 			     GTK_SIGNAL_FUNC(about_proc), 
-			     (gpointer) 0 );
+			     NULL );
   gtk_signal_connect_object( GTK_OBJECT(load_config), "activate",
 			     GTK_SIGNAL_FUNC(load_conf_proc), 
-			     (gpointer) 0 );
+			     NULL );
   gtk_signal_connect_object( GTK_OBJECT(save_config), "activate",
 			     GTK_SIGNAL_FUNC(save_conf_proc), 
-			     (gpointer) 0 );
+			     NULL );
   gtk_signal_connect_object( GTK_OBJECT(customize_entry), "activate",
 			     GTK_SIGNAL_FUNC(customize_entry_proc), 
-			     (gpointer) 0 );
+			     NULL );
   gtk_signal_connect_object( GTK_OBJECT(quit_menu), "activate",
 			     GTK_SIGNAL_FUNC(quit), 
-			     (gpointer) 0 );
+			     NULL );
 
   // and show the items:
   gtk_widget_show( about );
@@ -104,37 +111,92 @@ void fire_up_menu(GtkWidget* widget){
 /*}}}*/
 
 /*{{{  Menu handling subroutines  */
-static gint popup_handler(GtkWidget *widget, GdkEventButton *event, GtkWidget *menu){
+static gint 
+popup_handler(GtkWidget *widget, GdkEventButton *event, GtkWidget *menu)
+{
 
-  if (event->button == 3) {
+  if ( event->button == 2 )
+    {      
+      hide_show_buttons_proc();
+      
+      return TRUE;
+    }
 
-    gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
-
-    gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
-		    event->button, event->time);
-    return TRUE;
-  }
+  if (event->button == 3) 
+    { 
+      gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
+      
+      gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
+		      event->button, event->time);
+      return TRUE;
+    }
   return FALSE;
 }
 /*}}}*/
 
+/*{{{  hide_show_buttons procedure  */
+/* 
+   This function toggles the buttonbox in the main window to be
+   hidden or shown (toggle).
+   It keeps the lower border of the text scrollbar on the same relative position
+   to the end - everything else would probably be pretty annoying.
+*/
+void 
+hide_show_buttons_proc()
+{
+  GtkWidget* bbox = (GtkWidget*) gtk_object_get_data( GTK_OBJECT(window), BUTTONBOX);  
+
+  /* memorize old scrollbar position (lower border) relative to end */
+  GtkAdjustment* ver_text_adj = (GtkAdjustment*) ( GTK_RANGE(textscrollbar)->adjustment );
+  gfloat old_position = ver_text_adj->upper - ( ver_text_adj->value + ver_text_adj->page_size );
+  
+  if ( buttons_shown == TRUE )
+    {
+      gtk_widget_hide( bbox );
+      buttons_shown = FALSE;
+    } 
+  else 
+    {
+      gtk_widget_show( bbox );
+      buttons_shown = TRUE;
+    }
+  
+  gtk_widget_realize(window);
+  gtk_widget_realize(textwindow);
+  gtk_widget_map(textwindow);
+  gtk_widget_realize(textscrollbar);
+  gtk_widget_map(textscrollbar);
+
+  /* Now put scrollbar on same position relative to end */
+  gtk_adjustment_set_value(ver_text_adj, 
+			   ver_text_adj->upper - old_position - ver_text_adj->page_size);
+
+}
+/*}}}*/
+
 /*{{{  Load Configuration  */
-void load_conf_proc(GtkWidget* widget, gpointer* data){
-  if ( customizing == TRUE || filew != 0 ){    
-    GList* buttons = g_list_alloc();
-    GList* functions = g_list_alloc();
-    buttons = g_list_append(buttons,
-			    (gpointer) "\t   DISMISS   \t");
-    functions = g_list_append(functions, NULL );
-    
-    popup_dialog("Erratic Request", "\n\t    Unable to load configuration     \t\n\t     at this point in program !     \t\n", buttons, functions);
-    
-    return;
-  }
-
-  if ( configuration_changed == FALSE ||
-       terse == TRUE ) do_load();
-
+void 
+load_conf_proc(GtkWidget* widget, gpointer* data)
+{
+  if ( customizing == TRUE || filew != 0 )
+    {
+      GList* buttons = g_list_alloc();
+      GList* functions = g_list_alloc();
+      buttons = g_list_append(buttons,
+			      (gpointer) "\t   DISMISS   \t");
+      functions = g_list_append(functions, NULL );
+      
+      popup_dialog("Erratic Request", "\n\t    Unable to load configuration     \t\n\t     at this point in program !     \t\n", buttons, functions);
+      
+      return;
+    }
+  
+  if ( configuration_changed == FALSE || terse == TRUE )
+    {
+      do_load();
+      return;
+    }
+  
   /* the configuration has been changed really load ? */
   
   GList* buttons = g_list_alloc();
@@ -153,19 +215,22 @@ void load_conf_proc(GtkWidget* widget, gpointer* data){
      "\n\t     There are unsaved changes     \t\n\t     loading will cause a loss of changes     \t\n\n\t     Proceed ?     \t\n",
      buttons, functions);
 }
-void do_load(){
-  if ( customizing == TRUE || filew != 0 ){
-    
-    GList* buttons = g_list_alloc();
-    GList* functions = g_list_alloc();
-    buttons = g_list_append(buttons,
-			    (gpointer) "\t   DISMISS   \t");
-    functions = g_list_append(functions, NULL );
-    
-    popup_dialog("Erratic Request", "\n\t    Unable to load new configuration     \t\n\t     at this point in program execution !     \t\n", buttons, functions);
-    
-    return;
-  }
+void 
+do_load()
+{
+  if ( customizing == TRUE || filew != 0 )
+    {
+      
+      GList* buttons = g_list_alloc();
+      GList* functions = g_list_alloc();
+      buttons = g_list_append(buttons,
+			      (gpointer) "\t   DISMISS   \t");
+      functions = g_list_append(functions, NULL );
+      
+      popup_dialog("Erratic Request", "\n\t    Unable to load new configuration     \t\n\t     at this point in program execution !     \t\n", buttons, functions);
+      
+      return;
+    }
   
   deactivate();
   disable();
@@ -200,7 +265,9 @@ void do_load(){
   gtk_window_set_policy(GTK_WINDOW(filew), TRUE, TRUE, TRUE);
   gtk_widget_show(filew);
 }
-void file_ok_sel (GtkWidget *w, GtkFileSelection *fs){
+void 
+file_ok_sel (GtkWidget *w, GtkFileSelection *fs)
+{
   char* fname = 
     new char[strlen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)))+2];
   strcpy(fname,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
@@ -240,34 +307,38 @@ void file_cancel_sel (GtkWidget *w, GtkFileSelection *fs){
 /*}}}*/
 
 /*{{{  Save Configuration  */
-void save_conf_proc(GtkWidget* widget, gpointer* data){
-  if ( customizing == TRUE || filew != 0 ){
-    GList* buttons = g_list_alloc();
-    GList* functions = g_list_alloc();
-    buttons = g_list_append(buttons,
-			    (gpointer) "\t   DISMISS   \t");
-    functions = g_list_append(functions, NULL );
-    
-    popup_dialog("Erratic Request", "\n\t    Unable to save configuration     \t\n\t     at this point in program !     \t\n", buttons, functions);
-    
-    return;
-  }
-
+void 
+save_conf_proc(GtkWidget* widget, gpointer* data)
+{
+  if ( customizing == TRUE || filew != 0 )
+    {
+      GList* buttons = g_list_alloc();
+      GList* functions = g_list_alloc();
+      buttons = g_list_append(buttons,
+			      (gpointer) "\t   DISMISS   \t");
+      functions = g_list_append(functions, NULL );
+      
+      popup_dialog("Erratic Request", "\n\t    Unable to save configuration     \t\n\t     at this point in program !     \t\n", buttons, functions);
+      
+      return;
+    }
+  
   deactivate();
   disable();
-
-  if ( save_path == NULL ){
-    char* hdir = NULL;
-    hdir = getenv("HOME");
-    if ( hdir == NULL ) hdir = getenv("USER");
-    if ( hdir == NULL ) hdir = getenv("PWD");
-    if ( hdir == NULL ) hdir = ".";
-    save_path = new char[ strlen(hdir) + strlen(CONFIGFILE) + 2 ];
-    strcpy ( save_path, hdir );
-    strcat ( save_path, "/" );
-    strcat ( save_path, CONFIGFILE );
-  }
-
+  
+  if ( save_path == NULL )
+    {
+      char* hdir = NULL;
+      hdir = getenv("HOME");
+      if ( hdir == NULL ) hdir = getenv("USER");
+      if ( hdir == NULL ) hdir = getenv("PWD");
+      if ( hdir == NULL ) hdir = ".";
+      save_path = new char[ strlen(hdir) + strlen(CONFIGFILE) + 2 ];
+      strcpy ( save_path, hdir );
+      strcat ( save_path, "/" );
+      strcat ( save_path, CONFIGFILE );
+    }
+  
   /* Create a new file selection widget */
   filew = gtk_file_selection_new ("Save configuration");
   
@@ -295,7 +366,9 @@ void save_conf_proc(GtkWidget* widget, gpointer* data){
   gtk_window_position(GTK_WINDOW(filew), GTK_WIN_POS_MOUSE);
   gtk_widget_show(filew);
 }
-void file_save_ok (GtkWidget *w, GtkFileSelection *fs){
+void 
+file_save_ok (GtkWidget *w, GtkFileSelection *fs)
+{
   char* fname = 
     new char[strlen(gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)))+2];
   strcpy(fname,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
@@ -605,7 +678,7 @@ void retrieve_data(char** fname, char** btext, char** htext, int* interval, int*
   list_items = g_list_first(list_items);
 
   (*filter_tmp) = new Filter*[items+1];
-  for ( int z = 0 ; z < items ; z++ ){
+  for ( guint z = 0 ; z < items ; z++ ){
     GtkWidget* active_item = (GtkWidget*) list_items->data;
     Filter* filter = (Filter*) gtk_object_get_data( GTK_OBJECT( active_item ), FILTER_LIST_ITEM_DATA );
     (*filter_tmp)[z] = new Filter;
@@ -761,7 +834,6 @@ void form_add(GtkWidget *widget, gpointer *data){
 void form_move_up(GtkWidget *widget, gpointer *data){
   gint pageno;
   pageno = gtk_notebook_current_page(GTK_NOTEBOOK(notebook));
-  gint pages = g_list_length( (GList*) GTK_NOTEBOOK(notebook)->children );
 
   if ( pageno == 0 ) return;
 
@@ -926,6 +998,9 @@ void accept_it(GtkWidget*, gpointer*){
 /*}}}*/
 
 /*{{{  Filter Menu  */
+/*
+  This routine creates the filter menu part on each notebook page
+*/
 GtkWidget* 
 create_filtermenu(GtkWidget* box, Filter** filter)
 {
@@ -974,6 +1049,11 @@ create_filtermenu(GtkWidget* box, Filter** filter)
   gint mode = NO_CHANGE;
   if ( filter != NULL )
     if (filter[0] != NULL ) mode=filter[0]->mode;
+
+  /*
+    CLASS0 Filter buttons
+  */
+
   button = gtk_check_button_new_with_label ( "Raise");
   gtk_tooltips_set_tip(tooltips, button, "... the 'raising' mode... matching lines will be displayed in PRELIGHT color... (see gtkrc) ... ", "... the 'raising' mode... matching lines will be displayed in PRELIGHT color... (see gtkrc) ...");
   gtk_table_attach(GTK_TABLE(table), button, 0, 1, 2, 3,
@@ -1000,7 +1080,10 @@ create_filtermenu(GtkWidget* box, Filter** filter)
     gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
   gtk_object_set_data( GTK_OBJECT(box), FILTER_MODE_HIDE, (GtkWidget*) button);
   gtk_object_set_data( GTK_OBJECT(button), FILTER_BUTTON_MODE, (gpointer) HIDE );  
-  
+
+  /*
+    CLASS1 Filter buttons
+  */
 
   button = gtk_check_button_new_with_label ( "Alert");
   gtk_tooltips_set_tip(tooltips, button, "... the 'alert' mode, matching lines will trigger alert ... ", "... the 'alert' mode, matching lines will trigger alert ...");
@@ -1038,6 +1121,29 @@ create_filtermenu(GtkWidget* box, Filter** filter)
   gtk_object_set_data( GTK_OBJECT(box), FILTER_MODE_EXECUTE, (GtkWidget*) button);
   gtk_object_set_data( GTK_OBJECT(button), FILTER_BUTTON_MODE, (gpointer) EXECUTE );
 
+  /*
+    special mode buttons
+  */
+  button = gtk_check_button_new_with_label ( "Invert");
+  gtk_tooltips_set_tip(tooltips, button, "...invert sense of matching. If activated non-matching lines will trigger filter...", "...invert sense of matching. If activated non-matching lines will trigger filter...");
+  gtk_table_attach(GTK_TABLE(table), button, 2, 3, 0, 1,
+                   GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  if ( mode & INVERT )
+    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+  gtk_object_set_data( GTK_OBJECT(box), FILTER_MODE_INVERT, (GtkWidget*) button);
+  gtk_object_set_data( GTK_OBJECT(button), FILTER_BUTTON_MODE, (gpointer) INVERT );  
+  
+  button = gtk_check_button_new_with_label ( "Case-sensitive");
+  gtk_tooltips_set_tip(tooltips, button, "...if activated matching will be performed case-sensitive...", "...if activated matching will be performed case-sensitive...");
+  gtk_table_attach(GTK_TABLE(table), button, 1, 2, 0, 1,
+                   GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  if ( mode & CASE_SENSITIVE )
+    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+  else
+    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), FALSE);
+  gtk_object_set_data( GTK_OBJECT(box), FILTER_MODE_CASE_SENSITIVE, (GtkWidget*) button);
+  gtk_object_set_data( GTK_OBJECT(button), FILTER_BUTTON_MODE, (gpointer) CASE_SENSITIVE );  
+  
   /*
     Attach the signal handler AFTER everything is set up, otherwise
     it will call the function when stuff isn't set up properly...
@@ -1083,6 +1189,18 @@ create_filtermenu(GtkWidget* box, Filter** filter)
   gtk_signal_connect (GTK_OBJECT (button), "clicked",
 		      GTK_SIGNAL_FUNC (mode_change), (gpointer) box );
   gtk_widget_show (button);  
+
+  button = (GtkWidget*) 
+    gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_INVERT);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      GTK_SIGNAL_FUNC (mode_change), (gpointer) box );
+  gtk_widget_show (button);  
+
+  button = (GtkWidget*) 
+    gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_CASE_SENSITIVE);
+  gtk_signal_connect (GTK_OBJECT (button), "clicked",
+		      GTK_SIGNAL_FUNC (mode_change), (gpointer) box );
+  gtk_widget_show (button);  
   
 /*}}}*/
 
@@ -1090,17 +1208,19 @@ create_filtermenu(GtkWidget* box, Filter** filter)
   gchar* string = "";
   if ( filter != NULL )
     if (filter[0] != NULL ) string = filter[0]->string;
-  GtkWidget* label = gtk_label_new( " Pattern string:" );
+
+  GtkWidget* label = gtk_label_new( " REGEX:" );
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
-  gtk_table_attach(GTK_TABLE(table), label, 0, 3, 0, 1,
-                   GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-  gtk_widget_show(label);
+  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
+		   GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  //  gtk_widget_show(label);
+
   GtkWidget* text = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(text), string);
   gtk_entry_set_editable (GTK_ENTRY(text),TRUE);
   gtk_table_attach(GTK_TABLE(table), text, 0, 3, 1, 2,
                    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
-  gtk_tooltips_set_tip(tooltips, text, "... pattern string to search for ... may contain wildcards ...", "... pattern string to search for ... may contain wildcards ...");
+  gtk_tooltips_set_tip(tooltips, text, "...a standard REGEX (see GNU regex manual) to be used for this filter, matching lines will trigger the filter...", "...a standard REGEX (see GNU regex manual) to be used for this filter, matching lines will trigger the filter...");
   gtk_widget_show(text);
   gtk_object_set_data( GTK_OBJECT( box ), FILTER_STRING, (GtkWidget*) text);
   gtk_object_set_data( GTK_OBJECT( box ), FILTER_STRING_LABEL, (GtkWidget*) label);
@@ -1177,7 +1297,15 @@ create_filtermenu(GtkWidget* box, Filter** filter)
   gtk_widget_grab_focus(button);
   return list;
 }
-void mode_change(GtkWidget *widget, gpointer *data){
+/*
+  This function gets called when either the mode of the filter has been
+  changed or the string has been entered (RETURN has been hit in
+  GtkEntry) so basically every time the information needs to be
+  updated.
+*/
+void
+mode_change(GtkWidget *widget, gpointer *data)
+{
   if ( updating == TRUE ) return;
   gchar* string;
   gint mode;
@@ -1224,6 +1352,14 @@ void mode_change(GtkWidget *widget, gpointer *data){
     gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_EXECUTE);
   if (GTK_TOGGLE_BUTTON (button)->active) filter->mode |= EXECUTE;
 
+  button = (GtkWidget*) 
+    gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_INVERT);
+  if (GTK_TOGGLE_BUTTON (button)->active) filter->mode |= INVERT;
+  
+  button = (GtkWidget*) 
+    gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_CASE_SENSITIVE);
+  if (GTK_TOGGLE_BUTTON (button)->active) filter->mode |= CASE_SENSITIVE;
+
   /* now legalize the settings - there may only be one of the RAISE/LOWER/HIDES */
   filter->mode = legalize(filter->mode, clicked);
 
@@ -1242,6 +1378,16 @@ void mode_change(GtkWidget *widget, gpointer *data){
   position_child(list, active_item);
   gtk_list_select_child(GTK_LIST(list), active_item);
 
+  /*
+    Now test-calculate the REGEX to see whether it is correct
+    error reporting is done in the subroutine so we don't need to care
+    for it.
+  */
+  Filter* ftmp = new Filter;
+  ftmp->init(filter->string, filter->mode | COMPILE_REGEX );
+  delete ftmp;
+
+
   delete string;
   return;
 }
@@ -1355,7 +1501,7 @@ void position_child(GtkWidget* list, GtkWidget* child){
     gtk_object_get_data( GTK_OBJECT( child ), FILTER_LIST_ITEM_DATA );
   gint child_mode = filter->mode & CLASS0_MASK;
 
-  for ( gint x = 0 ; x < length ; x++ ){
+  for ( guint x = 0 ; x < length ; x++ ){
     GtkWidget* current_obj = (GtkWidget*) elements->data;
     filter = (Filter*) 
       gtk_object_get_data( GTK_OBJECT( current_obj ), FILTER_LIST_ITEM_DATA );
@@ -1418,6 +1564,16 @@ void update_filter_fields(Filter* filter, GtkWidget* box){
   if ( filter->mode & EXECUTE )
     gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
   else gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), FALSE);
+
+  button = (GtkWidget*) gtk_object_get_data( GTK_OBJECT( box ), FILTER_MODE_INVERT );
+  if ( filter->mode & INVERT )
+    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+  else gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), FALSE);
+
+  button = (GtkWidget*) gtk_object_get_data( GTK_OBJECT( box ), FILTER_MODE_CASE_SENSITIVE );
+  if ( filter->mode & CASE_SENSITIVE )
+    gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), TRUE);
+  else gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON (button), FALSE);
   
   GtkWidget* text = (GtkWidget*) gtk_object_get_data( GTK_OBJECT( box), FILTER_STRING );
   gtk_entry_set_text(GTK_ENTRY(text), filter->string);
@@ -1450,6 +1606,10 @@ void filter_set_sensitive(GtkWidget* box, gint sensitive){
 			   sensitive);
   gtk_widget_set_sensitive((GtkWidget*) gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_EXECUTE),
 			   sensitive);
+  gtk_widget_set_sensitive((GtkWidget*) gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_INVERT),
+			   sensitive);
+  gtk_widget_set_sensitive((GtkWidget*) gtk_object_get_data( GTK_OBJECT( box), FILTER_MODE_CASE_SENSITIVE),
+			   sensitive);
   gtk_widget_set_sensitive((GtkWidget*) gtk_object_get_data( GTK_OBJECT( box), FILTER_STRING),
 			   sensitive);
   gtk_widget_set_sensitive((GtkWidget*) gtk_object_get_data( GTK_OBJECT( box), FILTER_STRING_EXECUTE),
@@ -1489,6 +1649,16 @@ char* get_list_label(Filter* tmp){
   else label[x++] = ' ';
   
   label[x++] = ']';
+
+
+  label[x++] = '{';
+  if ( tmp->mode & CASE_SENSITIVE )
+    label[x++] = 'C';
+  if ( tmp->mode & INVERT )
+    label[x++] = 'I';
+  label[x++] = '}';
+
+
   label[x++] = 0;
   return label;
 }
@@ -1499,7 +1669,6 @@ char* merge_label(char* label1, char* label2){
 
   while (label1[y] != 0 ) result[x++] = label1[y++];
 
-  result[x++] = ' ';
   result[x++] = '[';
 
   y = 0;
