@@ -34,6 +34,7 @@
 
 #include "../config.h"
 #include "sysinc.H"
+#include "../import/import.h"
 #include "logclass.H"
 #include "xlogmaster.H"
 #include "extern.H"
@@ -44,7 +45,16 @@
 /*}}}*/
 
 /*{{{  Main  */
-int main (int argc, char *argv[]){
+/* 
+   this is the main function
+
+   it bascially only coordinates the startup and then hands over the control
+   to the GTK+ toolkit
+
+ */
+int 
+main (int argc, char *argv[])
+{
   /* Parse commandline & initialize toolkit */
   gtk_set_locale();
   gtk_init (&argc, &argv);
@@ -61,23 +71,38 @@ int main (int argc, char *argv[]){
 		      GTK_SIGNAL_FUNC (delete_event), NULL);
   gtk_widget_set_uposition( window, xpos, ypos );
 
-  /* create boxes */
-  box =  gtk_hbox_new(FALSE, 0);
+  /* create horizontally split box */
+  box = gtk_hbox_new (FALSE, 0);
   gtk_container_add(GTK_CONTAINER (window), box);
+  gtk_widget_show(box);
+
+  /*  initialize tooltips  */
+  tooltips = gtk_tooltips_new();
+
+  /* create textwidget */
+  /* if you want to know a few things about the setup of the main
+     window and why I don't use some things you'd like to have
+     please see comment on top of the create_textwidget function */
+  create_textwidget();
+
+  /* create scrolled box for buttons */
   buttonbox = gtk_vbox_new(FALSE, 0);
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, 
 				  GTK_POLICY_AUTOMATIC );
+  gtk_container_add (GTK_CONTAINER (scrolled_window), buttonbox);
+  gtk_object_set_data( GTK_OBJECT(window), BUTTONBOX, (GtkWidget*) scrolled_window );
+  
+  /* put box for buttons into window */
+  gtk_box_pack_start (GTK_BOX(box), scrolled_window, FALSE, FALSE, 0);
+  gtk_widget_show(scrolled_window);
+
+  /* set initial size of box for buttons */
   gtk_widget_set_usize(scrolled_window, 170, 0);
  
-  gtk_container_add (GTK_CONTAINER (scrolled_window), buttonbox);
-  gtk_widget_show(scrolled_window);
-  gtk_box_pack_end(GTK_BOX(box), scrolled_window, FALSE, FALSE, 5);
- 
-  /* create textwindow */
-  tooltips = gtk_tooltips_new();
-  create_textwidget();
+  /* Fire up Pop-Up Menu - the function is defined in menu.cc */
+  fire_up_menu(textwindow);
 
   /* do color stunts */
   color_init();
@@ -112,12 +137,22 @@ int main (int argc, char *argv[]){
 /*}}}*/
 
 /*{{{  GTK toolkit subroutines  */
-gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data){
+gint
+delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
+{
   // we will attach this to all windows so the window managers "delete" signal
   // won't kill our windows (that's what returning TRUE means)
+
+  /*
+    we do want the user whether he really wants to quit, though...
+  */
+  quit(widget, data);
+
   return (TRUE);
 }
-void quit (GtkWidget *widget, gpointer *data){
+void 
+quit (GtkWidget *widget, gpointer *data)
+{
   if ( terse == TRUE ){
     really_quit_button_pressed();
     return;
@@ -145,13 +180,17 @@ void quit (GtkWidget *widget, gpointer *data){
        "\n\t     Do you really want to quit ?     \t\n", 
        buttons, functions);
 }
-void really_quit_button_pressed(){
+void 
+really_quit_button_pressed()
+{
   disable();
   stop_watchdog();
   free_memory();
   gtk_main_quit();
 }
-void color_init(){
+void 
+color_init()
+{
   cmap = gdk_colormap_get_system();
   alert.pixel = 0;
   base.pixel = prelight.pixel = 0;
@@ -167,11 +206,33 @@ void color_init(){
   alert.green = (guint)( 65535.0 * c_alert[1] );
   alert.blue = (guint)( 65535.0 * c_alert[2] );
 }
-void create_textwidget(){
+/*
+  This routine is called during initalization and it creates the GtkText Widget.
+  
+  The return value is the box that contains the textwindow and it's scrollbar
+  so the box can be put into the paned window.
+*/
+void
+create_textwidget()
+{
   /*
+    NOTE ABOUT THE MAIN WINDOW CONSTRUCTION AND THE TEXT WINDOW
+    ESPECIALLY:
 
-    Currently the GtkText Widget does not support the horizontal scrollbar, so
-    this'll have to wait until it does...
+    The GTK+ toolkit still has some weaknesses and two of the main ones
+    affect the Xlogmaster (unfortunately):
+
+    - There is no horizontal scrollbar for the GtkText widget which means lines HAVE
+    to be wrapped, otherwise you'd lose information
+
+    - The paned widget (that would usually be the ultimate choice for the main window)
+    is rudimental at best - there is no handler for changing the aspect ratio, no way
+    totell it the RIGHT side should not get resized when resizing the main window
+    and so on... PLUS it is incredibly slow when resizing. Hopefully it'll work soon...
+
+
+    Just ignore this commented out code... I'll deal with it once the GTK+ toolkit
+    supplies the functionality I need,
 
     textwindow = gtk_text_new(NULL,NULL);
     gtk_text_set_editable(GTK_TEXT(textwindow), FALSE);
@@ -202,15 +263,19 @@ void create_textwidget(){
   gtk_text_set_editable(GTK_TEXT(textwindow), FALSE);
   gtk_text_set_word_wrap (GTK_TEXT(textwindow), wordwrap);
   textscrollbar = gtk_vscrollbar_new (GTK_TEXT(textwindow)->vadj);
-  gtk_box_pack_end(GTK_BOX(box), textscrollbar, FALSE, FALSE, 0);
-  gtk_box_pack_end(GTK_BOX(box), textwindow, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(box), textwindow, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(box), textscrollbar, FALSE, FALSE, 0);
   gtk_widget_show (textscrollbar);
   gtk_widget_show(textwindow);
-
-  /* Fire up Menu */
-  fire_up_menu(textwindow);
+ 
 }
-void create_buttons(){
+/*
+  This function creates the buttons - it is called at startup and during reinitialisation
+  when the user changed the config or loaded a new one...
+*/
+void 
+create_buttons()
+{
   for ( int i = 0 ; i < syslogs ; i++ ){
     entry[i]->button = gtk_button_new_with_label (entry[i]->buttontext);
     gtk_signal_connect (GTK_OBJECT (entry[i]->button), "clicked",
@@ -241,7 +306,9 @@ void about_ready(GtkWidget *widget, gpointer *pointer){
 /*}}}*/
 
 /*{{{  Data management  */
-void setup_logs(){
+void 
+setup_logs()
+{
   entry = new Log*[fillups];
 
   for ( int i = 0 ; i < fillups ; i++ ){
@@ -283,7 +350,7 @@ void setup_logs(){
       x = 0;
       while  ( f[x] != NULL ) {
 	entry[i]->filter[x] = new Filter();
-	entry[i]->filter[x]->init( f[x], m[x] );
+	entry[i]->filter[x]->init( f[x], m[x] | COMPILE_REGEX );
 	delete f[x];
 	entry[i]->filter[x]->execline = fe[x];
 	gint fclass = determine_filter_class(entry[i]->filter[x]->mode);
@@ -313,7 +380,9 @@ void setup_logs(){
   delete filter_mode;
   fillups = 0;
 }
-void unsetup_logs(){
+void 
+unsetup_logs()
+{
   for ( int i = 0 ; i < syslogs ; i++ ){
     remove_alert(i);                                              // clear alert routines & data
     gtk_widget_destroy(entry[i]->button);
@@ -323,7 +392,9 @@ void unsetup_logs(){
   syslogs = 0;
   entry = NULL;
 }
-void free_memory(){
+void 
+free_memory()
+{
   unsetup_logs();
   if ( load_path != NULL ) delete load_path;
   if ( save_path != NULL ) delete save_path;
@@ -339,15 +410,23 @@ void free_memory(){
 
   delete read_buffer;
   read_buffer = NULL;
+
+  delete case_fold;
 }
-void disable(){
+void 
+disable()
+{
   display_logs = DISABLED;
 }
-void enable(){
+void 
+enable()
+{
   display_logs = ENABLED;
 }
 /* routine to determine all filter classes applicable */
-gint determine_filter_class(gint mode){
+gint 
+determine_filter_class(gint mode)
+{
   int i;
   int fclass = NO_FILTER;
   
@@ -361,7 +440,9 @@ gint determine_filter_class(gint mode){
 
   return fclass;
 }
-void sort_filters(Filter** filter){
+void 
+sort_filters(Filter** filter)
+{
   if ( filter == NULL ) return;
   int amount = 0;
   while ( filter[amount] != NULL ) amount++;
@@ -392,6 +473,49 @@ void sort_filters(Filter** filter){
   delete tmp;
   return;
 }
+/*
+  This function disassembles the used REGEX buffers as
+  cleanly as possible. Since there is only one translation table
+  for all REGEX buffers this buffer is not freed.
+
+  Everything else is freed and set to zero before deleting the
+  REGEX buffer itself.
+  
+  I'd prefer to be able to use regfree, but it is a POSIX routine
+  and since it is commented out in rx.c it would only exist on
+  machines that do not need the libimport - making the whole
+  libimport obsolete... so I had to create this little function which
+  does exactly what regfree would have done... :-)
+*/
+void 
+disassemble_regex(re_pattern_buffer* regex)
+{
+  if ( regex != NULL )
+    {
+      /* set translate table to NULL */
+      regex->translate = NULL;
+      
+      /* free fastmap and set to zero if it exists */
+      if ( regex->fastmap != NULL )
+	{
+	  delete regex->fastmap;
+	  regex->fastmap = NULL;
+	  regex->fastmap_accurate = 0;
+	}
+      
+      /* free buffer and set to zero if it is used */
+      if ( regex->buffer != NULL && regex->allocated != 0 )
+	{
+	  free(regex->buffer);
+	  regex->buffer = NULL;
+	  regex->allocated = 0;
+	}
+    
+      delete regex;
+    }
+  
+  return;
+}
 /*}}}*/
 
 /*{{{  Initial preparations  */
@@ -525,80 +649,6 @@ get_value(char* string, int* index)
   else *index = y;
   return value;
 }
-/*
-  retrieves long commandline option (basically rebuilding the
-  functionality of getopt_long because that function doesn't exist
-  on all UN*X systems.)
-  parameters: argc, argv (as given to main), index: the position of
-  the last found parameter (or 0 if none)
-  return value: 
-  -1 = unknown parameter or no more parameters
-  -2 = "--" found - stop commandline parsing
-  otherwise amount of values retrieved (0 or 1)
-
-  the function parses the commandline and if it doesn't find the
-  commandline string in cmdline_options (see xlogmaster.H) it returns
-  -1. If it does find it, it looks at the next parameter (if existent) and
-  if it is unknown it is considered to be the value for the current one.
-
-*/
-gint
-my_getopt_long(int argc, char* argv[], int* index)
-{
-  cmdline_par = NULL;
-  cmdline_val = NULL;
-
-  if ( *index == 0 ) *index = 1;
-  if (  argv[*index] == NULL ||
-	*index >= argc ) return -1;
-  if ( ! strcmp( argv[*index], "--" )) return -2;
-  
-
-  gint x = getopt_sub( argv[*index] );
-  if ( x == -1 ) return -1;
-  
-  cmdline_par = cmdline_options[x];
-  
-  *index += 1;
-  
-  if (  argv[*index] == NULL ||
-	*index >= argc ) return 0;
-  
-  gint y = getopt_sub( argv[*index] );
-  if ( y != -1 ) return 0;
-  
-  cmdline_val = argv[*index];
-  *index += 1;
-  
-  return 1;
-}
-/* 
-   subroutine for my_getopt_long
-   
-   returns -1 if string could not be found in cmdline_options array
-   return position if string has been found
-*/
-gint
-getopt_sub(gchar* string)
-{
-  gint y = 0;
-  while ( string[y] != 0 &&
-	  string[y] == '-' ) y++; 
-  
-  if ( y == 0 || y > 2 ) return -1;          // only 1 or 2 '-' allowed
-
-  gint x = 0;
-  while ( cmdline_options[x] != NULL )
-    {
-      
-      if (  ! strcmp(&(string[y]), cmdline_options[x]) )
-	return x;
-      
-      x++;
-    }
-  
-  return -1;
-}
 /*}}}*/
 void 
 init(int argc,char* argv[])
@@ -619,6 +669,7 @@ init(int argc,char* argv[])
   notice_follows_mouse = FALSE; // no following around is default... most people will
   // find it too annoying.
   configuration_changed = FALSE; // no change of course...
+  buttons_shown = TRUE;             // the buttons are always shown at startup
 
   /* for execute.cc: */
   cleanup_tag = pid_reference = 0;
@@ -647,6 +698,19 @@ init(int argc,char* argv[])
   }
   eval_colorstring(cstring);
   delete cstring;
+
+
+  /*
+    set up translation table for case insensitivity
+  */
+  gint i;
+  case_fold = new gchar[257];
+  for ( i = 0 ; i < 256 ; i++ )
+    case_fold[i] = i;
+  for ( i = 'a' ; i <= 'z' ; i++ )
+    case_fold[i] = i - ( 'a' - 'A' );
+  case_fold[256] = 0;
+
 /*}}}*/
 
 /*{{{  set PATH variable  */
@@ -668,30 +732,32 @@ init(int argc,char* argv[])
 
 /*{{{  my own commandline things  */
  gint  x;
- gint opt_index = 0;
+ gint opt_index;
  int gtkrc = FALSE;
  terse = FALSE;
-  
- x = my_getopt_long(argc, argv, &opt_index);
- while ( x >= 0 ){
-   if ( ! strcmp( cmdline_par, "activate" ) )
+ 
+ x = getopt_long_only(argc, argv, "", options, &opt_index);
+ while ( x != -1 && x != '?' ){
+   const gchar* opt = options[opt_index].name;
+   
+   if ( ! strcmp( opt, "activate" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
+       if ( optarg == NULL ) usage();
        int index = 0;
-       request_active = get_value( cmdline_val, &index ) - 1 ;
+       request_active = get_value( optarg, &index ) - 1 ;
      } 
-   else if ( ! strcmp( cmdline_par, "alertcolor" ) )
+   else if ( ! strcmp( opt, "alertcolor" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
-       eval_colorstring(cmdline_val);
+       if ( optarg == NULL ) usage();
+       eval_colorstring(optarg);
      } 
-   else if ( ! strcmp( cmdline_par, "audio" ))
+   else if ( ! strcmp( opt, "audio" ))
      { 
-       if ( cmdline_val == NULL || x == 0 ) usage();
-       if ( ! strcmp( cmdline_val, "on" ) ) sound = TRUE;
-       if ( ! strcmp( cmdline_val, "ON" ) ) sound = TRUE;
-       if ( ! strcmp( cmdline_val, "off" ) ) sound = FALSE;
-       if ( ! strcmp( cmdline_val, "OFF" ) ) sound = FALSE;
+       if ( optarg == NULL ) usage();
+       if ( ! strcmp( optarg, "on" ) ) sound = TRUE;
+       if ( ! strcmp( optarg, "ON" ) ) sound = TRUE;
+       if ( ! strcmp( optarg, "off" ) ) sound = FALSE;
+       if ( ! strcmp( optarg, "OFF" ) ) sound = FALSE;
        if ( sound == FALSE ){
 	 if ( alert_sound != NULL ) delete alert_sound;
 	 if ( notice_sound != NULL ) delete notice_sound;
@@ -699,166 +765,163 @@ init(int argc,char* argv[])
 	 uniconify_sound = notice_sound = alert_sound = NULL;
        }
      }
-   else if ( ! strcmp( cmdline_par, "audio-alert" ) )
+   else if ( ! strcmp( opt, "audio-alert" ) )
      { 
-       if ( cmdline_val == NULL || x == 0 ) usage();
-       if ( ! strcmp( cmdline_val, "off" ) ) alert_sound = NULL;
-       else if ( ! strcmp( cmdline_val, "OFF" ) ) alert_sound = NULL;
+       if ( optarg == NULL ) usage();
+       if ( ! strcmp( optarg, "off" ) ) alert_sound = NULL;
+       else if ( ! strcmp( optarg, "OFF" ) ) alert_sound = NULL;
        else {
-	 alert_sound = new char[strlen(cmdline_val)+1];
-	 strcpy(alert_sound, cmdline_val);
+	 alert_sound = new char[strlen(optarg)+1];
+	 strcpy(alert_sound, optarg);
 	 sound = TRUE;
        }
      }
-   else if ( ! strcmp( cmdline_par, "audio-notice" ) )
+   else if ( ! strcmp( opt, "audio-notice" ) )
      { 
-       if ( cmdline_val == NULL || x == 0 ) usage();
-       if ( ! strcmp( cmdline_val, "off" ) ) notice_sound = NULL;
-       else if ( ! strcmp( cmdline_val, "OFF" ) ) notice_sound = NULL;
+       if ( optarg == NULL ) usage();
+       if ( ! strcmp( optarg, "off" ) ) notice_sound = NULL;
+       else if ( ! strcmp( optarg, "OFF" ) ) notice_sound = NULL;
        else {
-	 notice_sound = new char[strlen(cmdline_val)+1];
-	 strcpy(notice_sound, cmdline_val);
+	 notice_sound = new char[strlen(optarg)+1];
+	 strcpy(notice_sound, optarg);
 	 sound = TRUE;
        }
      } 
-   else if ( ! strcmp( cmdline_par, "audio-uniconify" ) )
+   else if ( ! strcmp( opt, "audio-uniconify" ) )
      { 
-       if ( cmdline_val == NULL || x == 0 ) usage();
-       if ( ! strcmp( cmdline_val, "off" ) ) uniconify_sound = NULL;
-       else if ( ! strcmp( cmdline_val, "OFF" ) ) uniconify_sound = NULL;
+       if ( optarg == NULL ) usage();
+       if ( ! strcmp( optarg, "off" ) ) uniconify_sound = NULL;
+       else if ( ! strcmp( optarg, "OFF" ) ) uniconify_sound = NULL;
        else {
-	 uniconify_sound = new char[strlen(cmdline_val)+1];
-	 strcpy(uniconify_sound, cmdline_val);
+	 uniconify_sound = new char[strlen(optarg)+1];
+	 strcpy(uniconify_sound, optarg);
 	 sound = TRUE;
        }
      } 
-   else if ( ! strcmp( cmdline_par, "buffer" ) )
+   else if ( ! strcmp( opt, "buffer" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
+       if ( optarg == NULL ) usage();
        int index = 0;
-       maxtext = get_value( cmdline_val, &index );
+       maxtext = get_value( optarg, &index );
      }
-   else if ( ! strcmp( cmdline_par, "config" ) )
+   else if ( ! strcmp( opt, "config" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
-       read_configuration(cmdline_val);
+       if ( optarg == NULL ) usage();
+       read_configuration(optarg);
      }
-   else if ( ! strcmp( cmdline_par, "fadeseconds" ) )
+   else if ( ! strcmp( opt, "fadeseconds" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
+       if ( optarg == NULL ) usage();
        int index = 0;
        int seconds = 0;
-       seconds = get_value( cmdline_val, &index );
+       seconds = get_value( optarg, &index );
        if ( seconds < 1 ) seconds = 1;
        fade_step = (-1.0 * ( (gdouble) fade_base / ( 10 * seconds )));
      }
-   else if ( ! strcmp( cmdline_par, "fadesteps" ) )
+   else if ( ! strcmp( opt, "fadesteps" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
+       if ( optarg == NULL ) usage();
        int index = 0;
-       steps = get_value( cmdline_val, &index );
+       steps = get_value( optarg, &index );
      }
-   else if ( ! strcmp( cmdline_par, "geometry" ) )
+   else if ( ! strcmp( opt, "geometry" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
+       if ( optarg == NULL ) usage();
        int index = 0;
-       char* string = cmdline_val;
+       char* string = optarg;
        
        if ( string[index] != 'x' )
-	 width = get_value( cmdline_val, &index );
+	 width = get_value( optarg, &index );
        if ( string[index] == 'x' ){
 	 index++;
-	 height = get_value( cmdline_val, &index );
+	 height = get_value( optarg, &index );
        }
        if ( string[index] == '+' ){
 	 index++;
-	 xpos = get_value( cmdline_val, &index );
+	 xpos = get_value( optarg, &index );
        }
        if ( string[index] == '+' ){
 	 index++;
-	 ypos = get_value( cmdline_val, &index );
+	 ypos = get_value( optarg, &index );
        }
      }
-   else if ( ! strcmp( cmdline_par, "gtkrc" ) )
+   else if ( ! strcmp( opt, "gtkrc" ) )
      {
-       if ( cmdline_val == NULL || x == 0 ) usage();
+       if ( optarg == NULL ) usage();
        gtkrc = TRUE;
-       gtk_rc_parse(cmdline_val);
+       gtk_rc_parse(optarg);
      }
-   else if  ( ! strcmp( cmdline_par, "help" ) )
+   else if  ( ! strcmp( opt, "help" ) )
      {
        help_message();
      }
 #if HAVE_MLOCKALL
-   else if ( ! strcmp( cmdline_par, "mlockall" ) )
+   else if ( ! strcmp( opt, "mlockall" ) )
      { 
-       if ( x == 1 ) usage();
        lockmem = TRUE;
      }
 #endif /* HAVE_MLOCKALL */
-   else if ( ! strcmp( cmdline_par, "notice-follows-mouse" ) )
+   else if ( ! strcmp( opt, "notice-follows-mouse" ) )
      { 
-       if ( cmdline_val == NULL )
+       if ( optarg == NULL )
 	 {
 	   notice_follows_mouse = TRUE;
 	 }
        else
 	 {
-	   if ( ! strcmp( cmdline_val, "yes" ) ) notice_follows_mouse = TRUE;
-	   if ( ! strcmp( cmdline_val, "YES" ) ) notice_follows_mouse = TRUE;
-	   if ( ! strcmp( cmdline_val, "no" ) ) notice_follows_mouse = FALSE;
-	   if ( ! strcmp( cmdline_val, "NO" ) ) notice_follows_mouse = FALSE;
+	   if ( ! strcmp( optarg, "yes" ) ) notice_follows_mouse = TRUE;
+	   if ( ! strcmp( optarg, "YES" ) ) notice_follows_mouse = TRUE;
+	   if ( ! strcmp( optarg, "no" ) ) notice_follows_mouse = FALSE;
+	   if ( ! strcmp( optarg, "NO" ) ) notice_follows_mouse = FALSE;
 	 }
      }
-   else if ( ! strcmp( cmdline_par, "silent" ) )
+   else if ( ! strcmp( opt, "silent" ) )
      { 
-       if ( x == 1 ) usage();
        sound = FALSE;
      }
-   else if ( ! strcmp( cmdline_par, "smallicon" ) )
+   else if ( ! strcmp( opt, "smallicon" ) )
      { 
-       if ( cmdline_val == NULL )
+       if ( optarg == NULL )
 	 {
 	   small_icon = TRUE;
 	 }
        else
 	 { 
-	   if ( ! strcmp( cmdline_val, "yes" ) ) small_icon = TRUE;
-	   if ( ! strcmp( cmdline_val, "YES" ) ) small_icon = TRUE;
-	   if ( ! strcmp( cmdline_val, "no" ) ) small_icon = FALSE;
-	   if ( ! strcmp( cmdline_val, "NO" ) ) small_icon = FALSE;
+	   if ( ! strcmp( optarg, "yes" ) ) small_icon = TRUE;
+	   if ( ! strcmp( optarg, "YES" ) ) small_icon = TRUE;
+	   if ( ! strcmp( optarg, "no" ) ) small_icon = FALSE;
+	   if ( ! strcmp( optarg, "NO" ) ) small_icon = FALSE;
 	 }
      }
-   else if ( ! strcmp( cmdline_par, "terse" ) )
+   else if ( ! strcmp( opt, "terse" ) )
      { 
-       if ( x == 1 ) usage();
        terse = TRUE;
      }
-   else if ( ! strcmp( cmdline_par, "version" ) )
+   else if ( ! strcmp( opt, "version" ) )
      {
        version_message();
      }
-   else if ( ! strcmp( cmdline_par, "wordwrap" ) )
+   else if ( ! strcmp( opt, "wordwrap" ) )
      { 
-       if ( cmdline_val == NULL )
+       if ( optarg == NULL )
 	 {
-	   wordwrap = TRUE;
+	   small_icon = TRUE;
 	 }
        else
 	 { 
-	   if ( ! strcmp( cmdline_val, "yes" ) ) wordwrap = TRUE;
-	   if ( ! strcmp( cmdline_val, "YES" ) ) wordwrap = TRUE;
-	   if ( ! strcmp( cmdline_val, "no" ) ) wordwrap = FALSE;
-	   if ( ! strcmp( cmdline_val, "NO" ) ) wordwrap = FALSE;
+	   if ( ! strcmp( optarg, "yes" ) ) wordwrap = TRUE;
+	   if ( ! strcmp( optarg, "YES" ) ) wordwrap = TRUE;
+	   if ( ! strcmp( optarg, "no" ) ) wordwrap = FALSE;
+	   if ( ! strcmp( optarg, "NO" ) ) wordwrap = FALSE;
 	 }
      }
    
-   x = my_getopt_long(argc, argv, &opt_index);
+   x = getopt_long_only(argc, argv, "", options, &opt_index);
  };
  
- if ( opt_index < argc && x != -2 )
+ if ( optind < argc || x == '?' )
    usage();
-
+ 
 /*}}}*/
 
 /*{{{  System-wide GTKRC  */
@@ -916,7 +979,7 @@ init(int argc,char* argv[])
     filter = new char**[fillups];
     filter_exec = new char**[fillups];
     filter_mode = new int*[fillups];
-    for ( int i = 0 ; i < fillups ; i++ ){
+    for ( i = 0 ; i < fillups ; i++ ){
       filename[i] = new char[50];
       help[i] = new char[50];  
       buttontext[i] = new char[50];
@@ -999,7 +1062,23 @@ init(int argc,char* argv[])
 /*}}}*/
 
 /*{{{  Text output subroutines  */
-void output_text(GtkWidget* target, char* buffer, long size){
+/*
+  This routine adds text at the current insertion point into
+  the GtkText widget while applying all CLASS0 Filters.
+
+  The parameters are:
+  o the GtkText Widget to use
+  o the buffer to insert
+  o the buffersize (buffer zero termination is neither used nor
+  checked)
+
+  the routine automagically applies all filters from the
+  "active" entry...
+
+*/
+void 
+output_text(GtkWidget* target, char* buffer, long size)
+{
   /*
     If no filter has been set we can just output and bail out
    */
@@ -1022,113 +1101,246 @@ void output_text(GtkWidget* target, char* buffer, long size){
   raise_bg = &style->bg[GTK_STATE_PRELIGHT];
   lower = &style->fg[GTK_STATE_INSENSITIVE];
   lower_bg = &style->bg[GTK_STATE_INSENSITIVE];
-  long start = 0;
-  long stop = start;
-  while ( stop < size ){
-    int action = NO_CHANGE;
-    int i = 0;
-    while ( entry[active]->filter[i] != NULL && action != HIDE ){
-      if ( determine_filter_class(entry[active]->filter[i]->mode) & CLASS0_FILTER )
-	if ( does_match(buffer, entry[active]->filter[i]->pattern,
-			stop, size ) == TRUE ) action |= entry[active]->filter[i]->mode;
-      
-      i++;
-    }
+  glong start = 0;
+  glong stop = start;
+  glong new_stop = stop;
 
-    action &= CLASS0_MASK;
+  while ( stop < size )
+    {
+   
+      new_stop = match_all( entry[active]->filter, CLASS0_FILTER, buffer, stop, size);
 
-    if ( action != NO_CHANGE ){
-      // first backtrace our steps until the beginning or the last CR / LF
-      while ( stop > start && buffer[stop] != '\n' && buffer[stop] != '\r' ) stop--;
-      // and now step over any CR/LF's there may be to avoid ugly blank
-      // lines when hiding them...
-      while ( stop < size && (  buffer[stop] == '\n' || buffer[stop] == '\r' ) ) stop++;
-      if ( start < stop )
-	gtk_text_insert (GTK_TEXT(target),NULL,NULL,NULL,
-			 (char*) (buffer+start), (stop-start));
-      start = stop;
-      stop++;
-      // now step forward until end of buffer or next CR / LF 
-      while ( stop < size && buffer[stop] != '\n' && buffer[stop] != '\r' ) stop++;
-
-      if ( ! ( action & HIDE ) ){
-	// we need to search once more for a possible HIDE override
-	for ( long n = start ; n < stop ; n++ ){
-	  i = 0;
-	  while ( entry[active]->filter[i] != NULL && action != HIDE ){
-	    if ( entry[active]->filter[i]->mode & HIDE )
-	      if ( does_match(buffer, entry[active]->filter[i]->pattern,
-			      n, size ) == TRUE )
-		action |= entry[active]->filter[i]->mode;
-	    i++;
-	  }
+      gint action = NO_CHANGE;
+      gint i = 0;
+      while ( entry[active]->filter[i] != NULL )
+	{
+	  
+	  if ( entry[active]->filter[i]->regex_status >= 0 )
+	    {
+	      action |= entry[active]->filter[i]->mode;
+	    }
+	  
+	  i++;
 	}
-      }
 
+      /*
+	mask out things that are not CLASS0_FILTERS,
+	afterwards actions has the bit for every applicable
+	CLASS0_FILTER set.
+       */
       action &= CLASS0_MASK;
 
-      if ( action & HIDE ){
-	// output nothing if we wanna hide - this is simple... 
-	// skip that silly NL - we don't want it...
-	while ( stop < size && (  buffer[stop] == '\n' || buffer[stop] == '\r' ) ) stop++;
-      } else if ( action & RAISE ){
-	gtk_text_insert (GTK_TEXT(target),NULL, raise, raise_bg,
-			 (char*) (buffer+start), (stop-start));
-      } else if ( action & LOWER ){
-	gtk_text_insert (GTK_TEXT(target),NULL, lower, lower_bg,
-			 (char*) (buffer+start), (stop-start));
-      }
 
-      start = stop;
-    } else stop++;
-  }
+      /*
+	if action is not NO_CHANGE we need to do something about
+	that line, so here is our routine for that:
+      */
+      
+      if ( action != NO_CHANGE )
+	{
+
+	  /* 
+	     first output the text until the matching line
+	     begins (if there is any)
+	  */
+	  if ( start < stop )
+	    {
+	      gtk_text_insert (GTK_TEXT(target),NULL,NULL,NULL,
+			       (char*) (buffer+start), (stop-start));
+	    }
+
+	  /*
+	    now set "start" and "stop" index to match the line
+	    we're just in...
+	  */
+	  start = stop;
+	  stop = new_stop;
+
+
+	  /*
+	    Now process the special line,
+	    the priority is HIDE over RAISE over LOWER
+	  */
+
+	  if ( action & HIDE )
+	    {
+	      
+	      /*
+		just do nothing....
+	      */
+	      
+	    } 
+	  else if ( action & RAISE )
+	    {
+	      
+	      gtk_text_insert (GTK_TEXT(target),NULL, raise, raise_bg,
+			       (char*) (buffer+start), (stop-start)+1);
+	      
+	    } 
+	  else if ( action & LOWER )
+	    {
+	      
+	      gtk_text_insert (GTK_TEXT(target),NULL, lower, lower_bg,
+			       (char*) (buffer+start), (stop-start)+1);
+	      
+	    }
+	  
+	  stop = new_stop + 1;
+	  start = stop;
+	}
+      
+      stop = new_stop + 1;
+      
+    }
+  
+  /*
+    output the remaining normal text (no special matches found)
+  */
   if ( start != stop )
     gtk_text_insert (GTK_TEXT(target),NULL,NULL,NULL,
 		     (char*) (buffer+start), (stop-start));
+  
+
 }
-int does_match(char* buffer, char* regexp, long index, long size){
-  // the regexp is already all in capitals... so we don't have to 
-  // worry about capitalizing it...
-  int i = 0;
-  char*  new_regexp;
-  while ( regexp[i] != 0 && (long) ( index + i ) < size ){
-    switch ( regexp[i] ){
-    case '*':
+/*}}}*/
 
-      new_regexp = (char*) (regexp+i+1);
-      while ( (long) ( index + i ) < size &&
-	      buffer[index + i] != 10 &&
-	      buffer[index + i] != 13 ){
-	if ( does_match(buffer, new_regexp , index + i, size) == TRUE )
-	  return TRUE;
-	i++;
-      }
-      return FALSE;
-     
-      break;
-    case '?':
+/*{{{  REGEX Matching Routines  */
+/*
+  REGEX match all filters on one line routine
 
-      i++;
-      break;
+  The matching result for each filter element is returned in it's
+  filter->regex_status (either zero or -1)
 
-    default:
+  The parameters for this function are:
+  o pointer to a NULL terminated array of filter elements.
+  o the filter class that is of interest for us.
+  o the pointer to the buffer
+  o position of line in buffer to start search at
+  o length of buffer ( the routine will also realize a zero termination,
+  though)
 
-      char a = buffer[index + i];
-      if ( a >= 'a' && a <= 'z' ) a-= 'a' - 'A';            // Capitalize
-      if ( a != regexp[i] ) return FALSE;
-      if ( a == '\n' || a == '\r' ) return FALSE;
-      i++;
+  the return value is the position of the next line in the buffer ( =size if the 
+  matching is complete)
 
-      break;
+*/
+glong
+match_all(Filter** filter, gint want_class, gchar* buffer, glong index, glong size)
+{
+  gchar* line;
+  glong end = index + 1;
+
+  /*
+    search for end of current line
+  */
+  while ( buffer[end] != 0 && end < size &&
+	  buffer[end] != '\n' && buffer[end] != '\r' ) end++;
+  /*
+    now go to beginning of next one
+  */
+  while ( buffer[end] != 0 && end < size &&
+	  buffer[end] == '\n' && buffer[end] == '\r' ) end++;
+  
+  /* allocate memory */
+  line = new gchar[ end - index + 5 ];
+
+  /*
+    copy buffer into line
+  */
+  gint x = 0;
+  while ( ( index + x ) < end )
+    {
+      line[x] = buffer[ index + x ];
+      x++;
     }
-  }
-  if ( regexp[i] == 0 ) return TRUE;
-  return FALSE;
+  line[x] = 0; // force zero termination
+   
+  /*
+    if we don't have a filter array we are done already...
+  */
+  if ( filter == NULL )
+    {
+      delete line;
+      return end;
+    }
+
+  /*
+    now start the matching
+  */
+  x = 0;
+  while ( filter[x] != NULL )
+    {
+      
+      /*
+	only match if the filter has an element of the desired class
+	otherwise it is -1 !
+      */
+      if ( determine_filter_class(filter[x]->mode) & want_class )
+	{
+	  filter[x]->regex_status = match_line(filter[x], line);
+	}
+      else
+	{
+	  filter[x]->regex_status = -1;
+	}
+
+      x++;
+    }
+
+  /*
+    alright, that's it... we're done. Free string element "line" and return
+    our end position.
+  */
+  delete line;
+  return end;
+}
+/*
+  REGEX line matching routine
+  
+  The matching result is the returned returned value
+  ( either zero for a successful match or -1 )
+  
+  the return glong value is the index position for the next line
+  the parameters are
+  o the filter entry that should be searched
+  o the line to search - needs to be zero terminated !!!
+  
+*/
+gint
+match_line(Filter* filter, gchar* line)
+{
+  if ( filter->regex_pattern == NULL )
+    {
+      return -1;
+    }
+
+  gint result = re_search( filter->regex_pattern,
+			   line,
+			   strlen( line ),
+			   0,
+			   strlen( line ),
+			   NULL );
+  
+  if ( filter->mode & INVERT )
+    {      
+      if ( result >= 0 )
+	result = -1;
+      else
+	result = 0; 
+    }
+  
+  return result;
 }
 /*}}}*/
 
 /*{{{  Error procedures  */
-void file_error(gchar* file){
+void 
+file_error(gchar* file)
+{
+  /* give stderr message */
+  cerr << PACKAGE << ": file access error. Could not open" << endl;
+  cerr << "  " << file << endl;
+
+  /* and now popup dialog */
   gchar* opener = "\n\t     Error while trying to access file     \t\n\n\t      ";
   gchar* closer = "      \t\n\n\t   please check path and permissions.     \t\n";
   gchar* text = new gchar[strlen(file) + strlen(opener) + strlen(closer) + 10 ];
@@ -1145,7 +1357,13 @@ void file_error(gchar* file){
   popup_dialog("File Access Error", text, buttons, functions);
   delete text;
 }
-void fork_error(){
+void 
+fork_error()
+{
+  /* give stderr message */
+  cerr << PACKAGE << ": fork error - could not spawn process." << endl;
+  
+  /* and now popup dialog */
   GList* buttons = g_list_alloc();
   GList* functions = g_list_alloc();
   buttons = g_list_append(buttons,
